package com.ibm.able.examples.conversation;

//====================================================================
// Imports
//====================================================================
import javax.agent.Payload;
import org.jagent.acr.AcrPayload;

import com.ibm.able.Able;
import com.ibm.able.conversation.MessageForm;

/**
 * This class represents the message-form used during the AMAB Conversation
 * by the buyer to indicate a selection from the Offer set provided by the seller
 * <p>
 * @see		com.ibm.able.conversation.MessageForm
 * @see		com.ibm.able.conversation.MessageFormFactory
 */

public class AmabSelectionMessageForm implements MessageForm 
{
	// =====================================
	// Class variables.
	// =====================================

	protected static final String NAME_KEY = "message-name";
	protected static final String SELECTION_KEY = "selection";


	// =====================================
	// Instance variables
	// =====================================


	String name = "";	// A property. Must be non-null. Start with a safe default.
	int selection = -1;	// The constituent data. Must be set before calling formMessage().

	// =====================================
	// Constructors
	// =====================================
	
	/**
	 * Default Constructor
	 */
	public AmabSelectionMessageForm() 
	{
		super();
	}


	/**
	 * Single Argument Constructor - as required by the MessageFormFactory
	 */
	public AmabSelectionMessageForm(String newName) 
	{
		super();
		if (newName != null)
			name = newName;
	}
	

	// =====================================
	// Properties
	// =====================================

	/**
	 * Return the name
	 * <p>
	 * @return		the name 
	 */
	public String getName() 
	{
		return name;
	}


	/**
	 * Set the name of the Selection Message
	 * <p>
	 * @param		newName
	 * 				the name
	 */
	public void setName(String newName) 
	{
		if (newName == null)
			name = "";
		else
			name = newName;
	}


	// =====================================
	// Constituent data.
	// =====================================

	/**
	 * Return the selection value
	 * <p>
	 * @return 	the selection value
	 */
	public int getSelection() 
	{
		return selection;
	}

	
	/**
	 * Set the selection value
	 * <p>
	 * @param		selection
	 * 				the actual selection value
	 */
	public void setSelection(int selection) 
	{
		this.selection = selection;
	}


	/**
	 * Tries to set the selection
	 * <p>
	 * @param data Length-1 array containing an Integer  as its first (and only) element.
	 */
	public void setData(Object[] data) throws IllegalArgumentException 
	{
		try 
		{
			Integer i = (Integer) data[0];
			selection = i.intValue();
		}
		catch (Exception e) 
		{
			throw new IllegalArgumentException("Bad data[0]: expected Integer.");
		}
	}


	/**
	 * Returns the selection, packed into a length-1 array. (To get the name, use getName()).
	 */
	public Object[] getData() 
	{
		return new Object[] { new Integer(selection) };
	}


	/**
	 * Sets selection to zero. Does NOT change name.
	 */
	public void clearData() 
	{
		selection = 0;
	}


	// =====================================
	// MessageForm methods
	// =====================================
	/**
	 * internal function to create a payload for contituent data
	 */
	protected Payload makePayload(String localName, Integer i) 
	{
		AcrPayload payload = new AcrPayload();
		payload.set(NAME_KEY, localName);
		payload.set(SELECTION_KEY, i);
		return payload;
	}


	/**
	 * @see MessageForm#formMessagePayload()
	 */
	public Payload formMessagePayload() throws IllegalStateException 
	{
		if (selection == -1)
			throw new IllegalStateException("Invalid selection choice");

		Payload newPayload = this.makePayload(name, new Integer(selection));
		return newPayload;
	}


	/**
	 * @see MessageForm#sampleMessagePayload()
	 */
	public Payload sampleMessagePayload() 
	{
		Payload newPayload = this.makePayload(name,new Integer(0));
		return newPayload;
	}


	/**
	 * @see MessageForm#parseMessagePayload(Payload)
	 */
	public boolean parseMessagePayload(Payload messagePayload) 
	{
		String newName = null;
		Integer i = null;
		try 
		{
			newName = (String) messagePayload.get(NAME_KEY);
			i = (Integer) messagePayload.get(SELECTION_KEY);
		} 
		catch (Exception ex) 
		{
			Able.MessageLog.text(Able.MSG_WARNING,this,"parseMessagePayload",ex.getLocalizedMessage());
        	if (Able.TraceLog.isLogging()) 
        		Able.TraceLog.exception(Able.TRC_LOW,this,"parseMessagePayload",ex);
		}

		boolean isParsed = false;
		if (newName != null && i != null) 
			isParsed = newName.equals(name);

		if (isParsed) 
			selection = i.intValue();

		return isParsed;
	}
}
